@@ -13,12 +13,12 @@ import subprocess
 import sys
 import tempfile
 import time
+from builtins import print as oprint
 from base64 import b64decode
 from concurrent.futures import ThreadPoolExecutor
 from copy import deepcopy
 from datetime import timedelta
 from glob import glob
-from importlib import metadata
 from multiprocessing import cpu_count
 from types import SimpleNamespace
 from typing import Any, NoReturn
@@ -44,35 +44,38 @@ from deew.messages import error_messages
 from deew.xml_base import xml_dd_ddp_base, xml_thd_base
 
 prog_name = 'deew'
-try:
-    prog_version = metadata.version('deew')
-except metadata.PackageNotFoundError:
-    pyproject_path = 'pyproject.toml'
-    is_pyinstaller = getattr(sys, 'frozen', False)
-    if is_pyinstaller:
-        pyproject_path = os.path.join(sys._MEIPASS, pyproject_path)
-    pyproject = toml.load(pyproject_path)
-    prog_version = pyproject['tool']['poetry']['version'] + ('' if is_pyinstaller else '-dev')
+prog_version = '2.8.3'
 
 simplens = SimpleNamespace()
 
+
 class RParse(argparse.ArgumentParser):
     def _print_message(self, message, file=None):
         if message:
             if message.startswith('usage'):
                 message = f'[bold cyan]{prog_name}[/bold cyan] {prog_version}\n\n{message}'
-                message = re.sub(r'(-[a-z]+\s*|\[)([A-Z]+)(?=]|,|\s\s|\s\.)', r'\1[{}]\2[/{}]'.format('color(231)', 'color(231)'), message)
+                message = re.sub(r'(-[a-z]+\s*|\[)([A-Z]+)(?=]|,|\s\s|\s\.)', r'\1[{}]\2[/{}]'.format('bold color(231)', 'bold color(231)'), message)
                 message = re.sub(r'((-|--)[a-z]+)', r'[{}]\1[/{}]'.format('green', 'green'), message)
                 message = message.replace('usage', f'[yellow]USAGE[/yellow]')
-                message = message.replace('options', f'[yellow]FLAGS[/yellow]')
+                message = message.replace('options', f'[yellow]FLAGS[/yellow]', 1)
                 message = message.replace(self.prog, f'[bold cyan]{self.prog}[/bold cyan]')
             message = f'[not bold white]{message.strip()}[/not bold white]'
             print(message)
 
 
+class CustomHelpFormatter(argparse.RawTextHelpFormatter):
+    def _format_action(self, action):
+        old = super()._format_action(action)
+        metavar = ''
+        m = re.search(r'--?[\w-]+ ([A-Z. \[\]]+)', old)
+        if m:
+            metavar = m.group(1)
+        return re.sub(r'(-[\w-]+) ([A-Z. \[\]]+), (--[\w-]+) \2', fr'\1, \3 \2{" " * (len(metavar) + 1)}', old)
+
+
 parser = RParse(
     add_help=False,
-    formatter_class=lambda prog: argparse.RawTextHelpFormatter(prog, width=80, max_help_position=40)
+    formatter_class=lambda prog: CustomHelpFormatter(prog, width=80, max_help_position=32)
 )
 parser.add_argument('-h', '--help',
                     action='help',
@@ -88,35 +91,61 @@ parser.add_argument('-i', '--input',
                     help='audio file(s) or folder(s)')
 parser.add_argument('-o', '--output',
                     default=None,
-                    help='output directory\ndefault: current directory')
+                    metavar='DIRECTORY',
+                    help='[underline magenta]default:[/underline magenta] current directory\nspecifies output directory')
 parser.add_argument('-f', '--format',
                     type=str,
                     default='ddp',
-                    help='dd / ddp / thd\ndefault: ddp')
+                    help=
+'''[underline magenta]options:[/underline magenta] [bold color(231)]dd[/bold color(231)] / [bold color(231)]ddp[/bold color(231)] / [bold color(231)]thd[/bold color(231)]
+[underline magenta]default:[/underline magenta] [bold color(231)]ddp[/bold color(231)]''')
 parser.add_argument('-b', '--bitrate',
                     type=int,
                     default=None,
-                    help='defaults: see config')
+                    help=
+'''[underline magenta]options:[/underline magenta] run [green]-lb[/green]/[green]--list-bitrates[/green]
+[underline magenta]default:[/underline magenta] run [green]-c[/green]/[green]--config[/green]''')
 parser.add_argument('-dm', '--downmix',
                     type=int,
                     default=None,
-                    help='1 / 2 / 6\nspecifies downmix, only works for DD/DDP\nDD will be automatically downmixed to 5.1 in case of a 7.1 source')
+                    metavar='CHANNELS',
+                    help=
+'''[underline magenta]options:[/underline magenta] [bold color(231)]1[/bold color(231)] / [bold color(231)]2[/bold color(231)] / [bold color(231)]6[/bold color(231)]
+specifies downmix, only works for DD/DDP
+DD will be automatically downmixed to 5.1 in case of a 7.1 source''')
 parser.add_argument('-d', '--delay',
                     type=str,
-                    default='+0ms',
-                    help='specifies delay as ms, s or frame@FPS\nFPS can be a number, division or ntsc / pal\n+ / - can also be defined as p / m\nexamples: -5.1ms, +1,52s, p5s, m5@pal, +10@24000/1001\ndefault: 0ms')
-parser.add_argument('-drc',
+                    default=None,
+                    help=
+'''[underline magenta]examples:[/underline magenta] [bold color(231)]-5.1ms[/bold color(231)], [bold color(231)]+1,52s[/bold color(231)], [bold color(231)]-24@pal[/bold color(231)], [bold color(231)]+10@24000/1001[/bold color(231)]
+[underline magenta]default:[/underline magenta] [bold color(231)]0ms[/bold color(231)] or parsed from filename
+specifies delay as ms, s or frame@FPS
+FPS can be a number, division or ntsc / pal
+you have to specify negative values as [bold color(231)]-[/bold color(231)][bold color(231)]d=-0ms[/bold color(231)]''')
+parser.add_argument('-r', '--drc',
                     type=str,
-                    default='film_light',
-                    help='film_light / film_standard / music_light / music_standard / speech\ndrc profile\ndefault: film_light')
+                    default='music_light',
+                    help=
+'''[underline magenta]options:[/underline magenta] [bold color(231)]film_light[/bold color(231)] / [bold color(231)]film_standard[/bold color(231)] / [bold color(231)]music_light[/bold color(231)] / [bold color(231)]music_standard[/bold color(231)] / [bold color(231)]speech[/bold color(231)]
+[underline magenta]default:[/underline magenta] [bold color(231)]music_light[/bold color(231)] (this is the closest to the missing none preset)
+specifies drc profile''')
 parser.add_argument('-dn', '--dialnorm',
                     type=int,
                     default=0,
-                    help='applied dialnorm value between -31 and 0\n0 means auto (DEE\'s measurement will be used)\ndefault: 0')
-parser.add_argument('-t', '--threads',
-                    type=int,
+                    help=
+'''[underline magenta]options:[/underline magenta] between [bold color(231)]-31[/bold color(231)] and [bold color(231)]0[/bold color(231)] (in case of [bold color(231)]0[/bold color(231)] DEE\'s measurement will be used)
+[underline magenta]default:[/underline magenta] [bold color(231)]0[/bold color(231)]
+applied dialnorm value between''')
+parser.add_argument('-in', '--instances',
+                    type=str,
                     default=None,
-                    help='number of threads to use, only works for batch encoding,\nindividial encodes can\'t be parallelized\n(this option overwrites the config\'s number)')
+                    help=
+'''[underline magenta]examples:[/underline magenta] [bold color(231)]1[/bold color(231)], [bold color(231)]4[/bold color(231)], [bold color(231)]50%%[/bold color(231)]
+[underline magenta]default:[/underline magenta] [bold color(231)]50%%[/bold color(231)]
+specifies how many encodes can run at the same time
+[bold color(231)]50%%[/bold color(231)] means [bold color(231)]4[/bold color(231)] on a cpu with 8 threads
+one DEE can use 2 threads so [bold color(231)]50%%[/bold color(231)] can utilize all threads
+(this option overwrites the config\'s number)''')
 parser.add_argument('-k', '--keeptemp',
                     action='store_true',
                     help='keep temp files')
@@ -206,12 +235,17 @@ dee_path = 'dee.exe'
 # You can also specify an absolute path or a path relative to the current directory.
 temp_path = ''
 
-logo = 1 # Set between 1 and 10, use the -pl/--print-logos option to see the available logos, set to 0 to disable logo.
-show_summary = true
+# Set between 1 and 10, use the -pl/--print-logos option to see the available logos, set to 0 to disable logo.
+logo = 1
 
-# You can overwrite this setting with -t/--threads. The number will be clamped between 1 and cpu_count().
-# Due to a DEE limitation it will be clamped between 1 and cpu_count() - 2 if you use the Windows version of DEE.
-threads = 10
+# Specifies how many encodes can run at the same time.
+# It can be a number or a % compared to your number of threads (so '50%' means 4 on an 8 thread cpu).
+# One DEE can use 2 threads so setting '50%' can utilize all threads.
+# You can overwrite this setting with -in/--instances.
+# The number will be clamped between 1 and cpu_count().
+# With the Windows version of DEE the max will be cpu_count() - 2 or 6 due to a limitation.
+# examples: 1, 4, '50%'
+max_instances = '50%'
 
 [default_bitrates]
     dd_1_0 = 128
@@ -220,7 +254,16 @@ threads = 10
     ddp_1_0 = 128
     ddp_2_0 = 256
     ddp_5_1 = 1024
-    ddp_7_1 = 1536'''
+    ddp_7_1 = 1536
+
+# You can toggle what sections you would like to see in the encoding summary
+[summary_sections]
+    deew_info = true
+    binaries = true
+    input_info = true
+    output_info = true
+    other = true
+'''
 
     if standalone:
         print(f'''Please choose config's location:
@@ -276,10 +319,10 @@ def parse_version_string(inp: list) -> str:
     return v
 
 def convert_delay_to_ms(inp, compensate):
-    if not inp.startswith(('-', '+', 'm', 'p')): print_exit('delay')
+    if not inp.startswith(('-', '+')): print_exit('delay')
     inp = inp.replace(',', '.')
 
-    negative = inp.startswith(('-', 'm'))
+    negative = inp.startswith(('-'))
 
     if '@' in inp:
         frame = round(float(re.sub('[^0-9\.]', '', inp.split('@')[0])))
@@ -335,14 +378,18 @@ def find_closest_allowed(value: int, allowed_values: list[int]) -> int:
 
 
 def wpc(p: str) -> str:
-    if simplens.dee_is_exe and platform.system() != 'Windows':
-        if not p.startswith('/mnt/'): print_exit('wsl_path', p)
-        parts = list(filter(None, p.split('/')))[1:]
-        parts[0] = parts[0].upper() + ':'
-        p = '\\'.join(parts) + '\\'
+    if not simplens.is_wsl: return p
+    if not p.startswith('/mnt/'): print_exit('wsl_path', p)
+    parts = list(filter(None, p.split('/')))[1:]
+    parts[0] = parts[0].upper() + ':'
+    p = '\\'.join(parts) + '\\'
     return p
 
 
+def rwpc(p: str) -> str:
+    return re.sub(r'^([a-z]):/', lambda m: f'/mnt/{m.group(1).lower()}/', p.replace('\\', '/'), flags=re.IGNORECASE)
+
+
 def save_xml(f: str, xml: dict[str, Any]) -> None:
     with open(f, 'w', encoding='utf-8') as fd:
         fd.write(xmltodict.unparse(xml, pretty=True, indent='  '))
@@ -421,7 +468,7 @@ def encode(task_id: TaskID, settings: list) -> None:
                     pb.update(task_id=task_id, completed=float(progress[1]))
 
             if 'error' in line.lower():
-                print(line.rstrip().split(': ', 1)[1])
+                oprint(line.rstrip().split(': ', 1)[1])
     pb.update(task_id=task_id, completed=100)
 
     if not args.keeptemp:
@@ -497,38 +544,47 @@ def main() -> None:
                     'dee_path',
                     'temp_path',
                     'logo',
-                    'show_summary',
-                    'threads',
-                    'default_bitrates'
+                    'max_instances',
+                    'default_bitrates',
+                    'summary_sections'
                 ]
     c_key_missing = []
     for c_key in config_keys:
         if c_key not in config: c_key_missing.append(c_key)
     if len(c_key_missing) > 0: print_exit('config_key', f'[bold yellow]{"[not bold white], [/not bold white]".join(c_key_missing)}[/bold yellow]')
 
-    if not config['temp_path']:
-        config['temp_path'] = os.path.join(script_path, 'temp') if standalone else tempfile.gettempdir()
-        if config['temp_path'] == '/tmp':
-            config['temp_path'] = '/var/tmp'
-    config['temp_path'] = os.path.abspath(config['temp_path'])
-    createdir(config['temp_path'])
-
     for i in config['dee_path'], config['ffmpeg_path'], config['ffprobe_path']:
         if not shutil.which(i): print_exit('binary_exist', i)
 
     with open(shutil.which(config['dee_path']), 'rb') as fd:
         simplens.dee_is_exe = fd.read(2) == b'\x4d\x5a'
+    simplens.is_wsl = simplens.dee_is_exe and platform.system() != 'Windows'
 
-    if args.threads:
-        threads = args.threads
+    if not config['temp_path']:
+        if simplens.is_wsl:
+            config['temp_path'] = rwpc(subprocess.run(['powershell.exe', "(gi $env:TEMP).fullname"], capture_output=True, encoding='utf-8').stdout.strip())
+        else:
+            config['temp_path'] = os.path.join(script_path, 'temp') if standalone else tempfile.gettempdir()
+            if config['temp_path'] == '/tmp':
+                config['temp_path'] = '/var/tmp'
+    config['temp_path'] = os.path.abspath(config['temp_path'])
+    createdir(config['temp_path'])
+
+    cpu__count = cpu_count()
+    if args.instances:
+        instances = args.instances
     else:
-        threads = config['threads']
-
+        instances = config['max_instances']
+    if isinstance(instances, str) and instances.endswith('%'):
+        instances = cpu__count * (int(instances.replace('%', '')) / 100)
+    else:
+        instances = int(instances)
     if simplens.dee_is_exe:
-        threads = clamp(threads, 1, cpu_count() - 2)
+        instances = clamp(instances, 1, cpu__count - 2)
+        instances = clamp(instances, 1, 6)
     else:
-        threads = clamp(threads, 1, cpu_count())
-    if threads == 0: threads = 1
+        instances = clamp(instances, 1, cpu__count)
+    if instances == 0: instances = 1
 
     aformat = args.format.lower()
     bitrate = args.bitrate
@@ -664,58 +720,63 @@ def main() -> None:
         xml_base['job_config']['filter']['audio']['pcm_to_ddp']['drc']['line_mode_drc_profile'] = args.drc
         xml_base['job_config']['filter']['audio']['pcm_to_ddp']['drc']['rf_mode_drc_profile'] = args.drc
         xml_base['job_config']['filter']['audio']['pcm_to_ddp']['custom_dialnorm'] = args.dialnorm
-        delay_print, delay_xml, delay_mode = convert_delay_to_ms(args.delay, compensate=True)
-        xml_base['job_config']['filter']['audio']['pcm_to_ddp'][delay_mode] = delay_xml
     elif aformat == 'thd':
         xml_base = xmltodict.parse(xml_thd_base)
         xml_base['job_config']['output']['mlp']['storage']['local']['path'] = f'\"{wpc(output)}\"'
         xml_base['job_config']['filter']['audio']['encode_to_dthd']['atmos_presentation']['drc_profile'] = args.drc
         xml_base['job_config']['filter']['audio']['encode_to_dthd']['presentation_8ch']['drc_profile'] = args.drc
         xml_base['job_config']['filter']['audio']['encode_to_dthd']['presentation_6ch']['drc_profile'] = args.drc
+        xml_base['job_config']['filter']['audio']['encode_to_dthd']['presentation_2ch']['drc_profile'] = args.drc
         xml_base['job_config']['filter']['audio']['encode_to_dthd']['custom_dialnorm'] = args.dialnorm
-        delay_print, delay_xml, delay_mode = convert_delay_to_ms(args.delay, compensate=False)
-        xml_base['job_config']['filter']['audio']['encode_to_dthd'][delay_mode] = delay_xml
-
     xml_base['job_config']['input']['audio']['wav']['storage']['local']['path'] = f'\"{wpc(config["temp_path"])}\"'
     xml_base['job_config']['misc']['temp_dir']['path'] = f'\"{wpc(config["temp_path"])}\"'
 
-    if config['show_summary']:
-        try:
-            r = requests.get('https://api.github.com/repos/pcroland/deew/releases/latest')
-            latest_version = json.loads(r.text)['tag_name']
-            if version.parse(prog_version.replace('-dev', '')) < version.parse(latest_version):
-                latest_version = f'[bold green]{latest_version}[/bold green] !!!'
-        except Exception:
-            latest_version = "[red]couldn't fetch"
-
-        simplens.dee_version = parse_version_string([config['dee_path']])
-        simplens.ffmpeg_version = parse_version_string([config['ffmpeg_path'], '-version'])
-        simplens.ffprobe_version = parse_version_string([config['ffprobe_path'], '-version'])
+    simplens.dee_version = parse_version_string([config['dee_path']])
+    simplens.ffmpeg_version = parse_version_string([config['ffmpeg_path'], '-version'])
+    simplens.ffprobe_version = parse_version_string([config['ffprobe_path'], '-version'])
 
+    if not all(x is False for x in config["summary_sections"].values()):
         summary = Table(title='Encoding summary', title_style='not italic bold magenta', show_header=False)
         summary.add_column(style='green')
         summary.add_column(style='color(231)')
 
-        summary.add_row('[bold cyan]Version', prog_version.replace('-dev', ''))
-        summary.add_row('[bold cyan]Latest', latest_version, end_section=True)
+        if config['summary_sections']['deew_info']:
+            try:
+                r = requests.get('https://api.github.com/repos/pcroland/deew/releases/latest')
+                latest_version = json.loads(r.text)['tag_name']
+                if version.parse(prog_version) < version.parse(latest_version):
+                    latest_version = f'[bold green]{latest_version}[/bold green] !!!'
+            except Exception:
+                latest_version = "[red]couldn't fetch"
+            summary.add_row('[bold cyan]Version', prog_version)
+            summary.add_row('[bold cyan]Latest', latest_version, end_section=True)
 
-        summary.add_row('[cyan]DEE version', simplens.dee_version)
-        summary.add_row('[cyan]ffmpeg version', simplens.ffmpeg_version)
-        summary.add_row('[cyan]ffprobe version', simplens.ffprobe_version, end_section=True)
+        if config['summary_sections']['binaries']:
+            summary.add_row('[cyan]DEE version', simplens.dee_version)
+            summary.add_row('[cyan]ffmpeg version', simplens.ffmpeg_version)
+            summary.add_row('[cyan]ffprobe version', simplens.ffprobe_version, end_section=True)
 
-        summary.add_row('[bold yellow]Output')
-        summary.add_row('Format', 'TrueHD' if aformat == 'thd' else aformat.upper())
-        summary.add_row('Channels', channel_number_to_name(outchannels))
-        summary.add_row('Bitrate', 'N/A' if aformat == 'thd' else f'{str(bitrate)} kbps')
-        summary.add_row('Dialnorm', 'auto (0)' if args.dialnorm == 0 else f'{str(args.dialnorm)} dB', end_section=True)
+        if config['summary_sections']['input_info']:
+            summary.add_row('[bold yellow]Input')
+            summary.add_row('Channels', channel_number_to_name(channels))
+            summary.add_row('Bit depth', str(bit_depth), end_section=True)
+
+        if config['summary_sections']['output_info']:
+            summary.add_row('[bold yellow]Output')
+            summary.add_row('Format', 'TrueHD' if aformat == 'thd' else aformat.upper())
+            summary.add_row('Channels', channel_number_to_name(outchannels))
+            summary.add_row('Bitrate', 'N/A' if aformat == 'thd' else f'{str(bitrate)} kbps')
+            summary.add_row('Dialnorm', 'auto (0)' if args.dialnorm == 0 else f'{str(args.dialnorm)} dB', end_section=True)
+
+        if config['summary_sections']['other']:
+            if args.delay:
+                delay_print, delay_xml, delay_mode = convert_delay_to_ms(args.delay, compensate=False)
+            summary.add_row('[bold yellow]Other')
+            summary.add_row('Files', str(len(filelist)))
+            summary.add_row('Max instances', str(f'{instances:g}'))
+            summary.add_row('Delay', delay_print if args.delay else '0 ms or parsed from filename')
+            summary.add_row('Temp path', config['temp_path'])
 
-        summary.add_row('[bold yellow]Input')
-        summary.add_row('Channels', channel_number_to_name(channels))
-        summary.add_row('Bit depth', str(bit_depth), end_section=True)
-        summary.add_row('[bold yellow]Other')
-        summary.add_row('Files', str(len(filelist)))
-        summary.add_row('Max threads', str(threads))
-        summary.add_row('Delay', delay_print)
         print(summary)
         print()
 
@@ -747,7 +808,7 @@ def main() -> None:
         channel_swap_args = []
         channel_swap_args_print = ''
 
-    if simplens.dee_is_exe and platform.system() != 'Windows':
+    if simplens.is_wsl:
         dee_xml_input_base = wpc(config['temp_path'])
     else:
         dee_xml_input_base = config['temp_path'] if config['temp_path'].endswith('/') else f'{config["temp_path"]}/'
@@ -779,6 +840,16 @@ def main() -> None:
         dee_print_list.append(dee_args_print)
         if intermediate_exists: intermediate_exists_list.append(filelist[i])
 
+        delay_in_filename = re.match(r'.+DELAY ([-|+]?[0-9]+m?s)\..+', filelist[i])
+        if delay_in_filename:
+            delay = delay_in_filename[1]
+            if not delay.startswith(('-', '+')):
+                delay = f'+{delay}'
+        else:
+            delay = '+0ms'
+        if args.delay:
+            delay = args.delay
+
         xml = deepcopy(xml_base)
         xml['job_config']['input']['audio']['wav']['file_name'] = f'\"{basename(filelist[i], "wav")}\"'
         if aformat == 'ddp':
@@ -795,6 +866,14 @@ def main() -> None:
             del xml['job_config']['output']['ec3']
         else:
             xml['job_config']['output']['mlp']['file_name'] = f'\"{basename(filelist[i], "thd")}\"'
+
+        if aformat in ['dd', 'ddp']:
+            delay_print, delay_xml, delay_mode = convert_delay_to_ms(delay, compensate=True)
+            xml['job_config']['filter']['audio']['pcm_to_ddp'][delay_mode] = delay_xml
+        else:
+            delay_print, delay_xml, delay_mode = convert_delay_to_ms(delay, compensate=False)
+            xml['job_config']['filter']['audio']['encode_to_dthd'][delay_mode] = delay_xml
+
         save_xml(os.path.join(config['temp_path'], sanitize_xml_name(basename(filelist[i], 'xml'))), xml)
 
         settings.append([filelist[i], output, length_list[i], ffmpeg_args, dee_args, intermediate_exists, aformat])
@@ -815,11 +894,14 @@ def main() -> None:
 
     signal.signal(signal.SIGINT, signal.SIG_DFL)
 
+    jobs = []
     with pb:
-        with ThreadPoolExecutor(max_workers=threads) as pool:
+        with ThreadPoolExecutor(max_workers=instances) as pool:
             for setting in settings:
                 task_id = pb.add_task('', visible=False, total=None)
-                pool.submit(encode, task_id, setting)
+                jobs.append(pool.submit(encode, task_id, setting))
+    for job in jobs:
+        job.result()
 
 
 if __name__ == '__main__':
